# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["ToolContextAwareSplittingResponse"]


class ToolContextAwareSplittingResponse(BaseModel):
    chunks: List[str]
    """
    An array of text chunks generated by splitting the input text based on the
    specified strategy.
    """
